@@ -2,40 +2,43 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 
-from trytond.model import ModelSQL, ModelView, fields
-from trytond.pyson import Eval
+from sql import Null
+
+from trytond.model import ModelSQL, ModelView, ValueMixin, fields
+from trytond.modules.currency.fields import Monetary
+from trytond.modules.product import price_digits, round_price
 from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval, TimeDelta
 from trytond.transaction import Transaction
-from trytond.modules.company.model import CompanyValueMixin
-from trytond.modules.product import price_digits
+
+default_lead_time = fields.TimeDelta(
+    "Default Lead Time",
+    domain=['OR',
+        ('default_lead_time', '=', None),
+        ('default_lead_time', '>=', TimeDelta()),
+        ],
+    help="The time from confirming the sales order to sending the "
+    "products.\n"
+    "Used for products without a lead time.")
 
 
 class Configuration(metaclass=PoolMeta):
     __name__ = 'product.configuration'
 
-    default_lead_time = fields.MultiValue(
-        fields.TimeDelta(
-            "Default Lead Time",
-            help="The time from confirming the sales order to sending the "
-            "products.\n"
-            "Used for products without a lead time."))
-
-    @classmethod
-    def default_default_lead_time(cls, **pattern):
-        return datetime.timedelta(0)
+    default_lead_time = fields.MultiValue(default_lead_time)
 
 
-class DefaultLeadTime(ModelSQL, CompanyValueMixin):
+class DefaultLeadTime(ModelSQL, ValueMixin):
     "Product Default Lead Time"
     __name__ = 'product.configuration.default_lead_time'
-    default_lead_time = fields.TimeDelta("Default Lead Time")
+    default_lead_time = default_lead_time
 
 
 class Template(metaclass=PoolMeta):
     __name__ = 'product.template'
     salable = fields.Boolean("Salable")
     sale_uom = fields.Many2One(
-        'product.uom', "Sale UOM",
+        'product.uom', "Sale UoM",
         states={
             'invisible': ~Eval('salable', False),
             'required': Eval('salable', False),
@@ -43,40 +46,28 @@ class Template(metaclass=PoolMeta):
         domain=[
             ('category', '=', Eval('default_uom_category')),
             ],
-        depends=['salable', 'default_uom_category'])
-    lead_time = fields.TimeDelta(
-        "Lead Time",
-        states={
-            'invisible': ~Eval('salable', False),
-            },
-        depends=['salable'],
-        help="The time from confirming the sales order to sending the "
-        "products.\n"
-        "If empty the default lead time from the configuration is used.")
+        help="The default Unit of Measure for sales.")
+    lead_time = fields.MultiValue(fields.TimeDelta(
+            "Lead Time",
+            domain=['OR',
+                ('lead_time', '=', None),
+                ('lead_time', '>=', TimeDelta()),
+                ],
+            states={
+                'invisible': ~Eval('salable', False),
+                },
+            help="The time from confirming the sales order to sending the "
+            "products.\n"
+            "If empty the default lead time from the configuration is used."))
+    lead_times = fields.One2Many(
+        'product.lead_time', 'template', "Lead Times")
 
     @classmethod
-    def __register__(cls, module_name):
-        transaction = Transaction()
-        cursor = transaction.connection.cursor()
-        update = transaction.connection.cursor()
-        table = cls.__table_handler__(module_name)
-        sql_table = cls.__table__()
-
-        super(Template, cls).__register__(module_name)
-
-        # Migration from 3.8: change delivery_time into timedelta lead_time
-        if table.column_exist('delivery_time'):
-            cursor.execute(*sql_table.select(
-                    sql_table.id, sql_table.delivery_time))
-            for id_, delivery_time in cursor:
-                if delivery_time is None:
-                    continue
-                lead_time = datetime.timedelta(days=delivery_time)
-                update.execute(*sql_table.update(
-                        [sql_table.lead_time],
-                        [lead_time],
-                        where=sql_table.id == id_))
-            table.drop_column('delivery_time')
+    def multivalue_model(cls, field):
+        pool = Pool()
+        if field == 'lead_time':
+            return pool.get('product.lead_time')
+        return super().multivalue_model(field)
 
     @fields.depends('default_uom', 'sale_uom', 'salable')
     def on_change_default_uom(self):
@@ -99,10 +90,43 @@ class Template(metaclass=PoolMeta):
                     })]
 
 
+class ProductLeadTime(ModelSQL, ValueMixin):
+    "Product Lead Time"
+    __name__ = 'product.lead_time'
+
+    template = fields.Many2One(
+        'product.template', "Template", ondelete='CASCADE')
+    lead_time = fields.TimeDelta(
+        "Lead Time",
+        domain=['OR',
+            ('lead_time', '=', None),
+            ('lead_time', '>=', TimeDelta()),
+            ])
+
+    @classmethod
+    def __register__(cls, module_name):
+        pool = Pool()
+        Template = pool.get('product.template')
+        template = Template.__table__()
+        table = cls.__table__()
+
+        super().__register__(module_name)
+
+        cursor = Transaction().connection.cursor()
+        template_h = Template.__table_handler__(module_name)
+        if template_h.column_exist('lead_time'):
+            cursor.execute(*table.insert(
+                    columns=[table.template, table.lead_time],
+                    values=template.select(
+                        template.id, template.lead_time,
+                        where=template.lead_time != Null)))
+            template_h.drop_column('lead_time')
+
+
 class Product(metaclass=PoolMeta):
     __name__ = 'product.product'
 
-    sale_price_uom = fields.Function(fields.Numeric(
+    sale_price_uom = fields.Function(Monetary(
             "Sale Price", digits=price_digits), 'get_sale_price_uom')
 
     @classmethod
@@ -123,41 +147,45 @@ class Product(metaclass=PoolMeta):
         '''
         pool = Pool()
         Uom = pool.get('product.uom')
-        User = pool.get('res.user')
+        Company = pool.get('company.company')
         Currency = pool.get('currency.currency')
         Date = pool.get('ir.date')
 
-        today = Date.today()
+        context = Transaction().context
         prices = {}
 
         assert len(products) == len(set(products)), "Duplicate products"
 
         uom = None
-        if Transaction().context.get('uom'):
-            uom = Uom(Transaction().context.get('uom'))
+        if context.get('uom'):
+            uom = Uom(context['uom'])
 
         currency = None
         if Transaction().context.get('currency'):
             currency = Currency(Transaction().context.get('currency'))
-
-        user = User(Transaction().user)
+        company = None
+        if context.get('company'):
+            company = Company(context['company'])
+        date = context.get('sale_date') or Date.today()
 
         for product in products:
-            prices[product.id] = product._get_sale_unit_price(
-                quantity=quantity)
-            if uom and product.default_uom.category == uom.category:
-                prices[product.id] = Uom.compute_price(
-                    product.default_uom, prices[product.id], uom)
-            else:
-                prices[product.id] = Uom.compute_price(
-                    product.default_uom, prices[product.id], product.sale_uom)
-            if currency and user.company:
-                if user.company.currency != currency:
-                    date = Transaction().context.get('sale_date') or today
+            unit_price = product._get_sale_unit_price(quantity=quantity)
+            if unit_price is not None:
+                if uom and product.default_uom.category == uom.category:
+                    unit_price = Uom.compute_price(
+                        product.default_uom, unit_price, uom)
+                elif product.sale_uom:
+                    unit_price = Uom.compute_price(
+                        product.default_uom, unit_price, product.sale_uom)
+            if currency and company and unit_price is not None:
+                if company.currency != currency:
                     with Transaction().set_context(date=date):
-                        prices[product.id] = Currency.compute(
-                            user.company.currency, prices[product.id],
+                        unit_price = Currency.compute(
+                            company.currency, unit_price,
                             currency, round=False)
+            if unit_price is not None:
+                unit_price = round_price(unit_price)
+            prices[product.id] = unit_price
         return prices
 
     @property
@@ -178,7 +206,8 @@ class Product(metaclass=PoolMeta):
         Date = Pool().get('ir.date')
 
         if not date:
-            date = Date.today()
+            with Transaction().set_context(context=self._context):
+                date = Date.today()
 
         lead_time = self.lead_time_used
         if lead_time is None:
@@ -200,7 +229,7 @@ class SaleContext(ModelView):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     sale_date = fields.Date("Sale Date")
     quantity = fields.Float("Quantity")
 
