@@ -1625,18 +1625,18 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
     def get_from_location(self, name):
         if (self.quantity or 0) >= 0:
             if self.warehouse:
-                return self.warehouse.output_location.id
+                return self.warehouse.output_location
         else:
             party = self.sale.shipment_party or self.sale.party
-            return party.customer_location.id
+            return party.customer_location
 
     def get_to_location(self, name):
         if (self.quantity or 0) >= 0:
             party = self.sale.shipment_party or self.sale.party
-            return party.customer_location.id
+            return party.customer_location
         else:
             if self.warehouse:
-                return self.warehouse.input_location.id
+                return self.warehouse.input_location
 
     @fields.depends('moves', methods=['planned_shipping_date'])
     def on_change_with_shipping_date(self, name=None):
@@ -1803,9 +1803,12 @@ class SaleLine(TaxableMixin, sequence_ordered(), ModelSQL, ModelView):
             if invoice_line.type != 'line':
                 continue
             if invoice_line not in skips:
-                quantity += Uom.compute_qty(
-                    invoice_line.unit or self.unit, invoice_line.quantity,
-                    self.unit)
+                if self.unit:
+                    quantity += Uom.compute_qty(
+                        invoice_line.unit or self.unit, invoice_line.quantity,
+                        self.unit)
+                else:
+                    quantity += invoice_line.quantity
         return quantity
 
     def _get_invoice_line_moves(self):
